==> magic-class-0.5.16.tar.gz: magic-class-0.5.16/magicclass/widgets/utils.py <==

from __future__ import annotations
from functools import wraps
from qtpy.QtWidgets import QWidget, QVBoxLayout, QGridLayout, QHBoxLayout
from magicgui.widgets import Widget
from magicgui.backends._qtpy.widgets import QBaseWidget

class FreeWidget(Widget):
    """
    A Widget class with any QWidget as a child.
    """    
    def __init__(self, layout="vertical", **kwargs):
        super().__init__(widget_type=QBaseWidget, backend_kwargs={"qwidg": QWidget}, **kwargs)
        self.native: QWidget
        self.central_widget: QWidget
        if layout == "vertical":
            self.native.setLayout(QVBoxLayout())
        elif layout == "horizontal":
            self.native.setLayout(QHBoxLayout())
        elif layout == "grid":
            self.native.setLayout(QGridLayout())
        else:
            ValueError(layout)
        self.native.setContentsMargins(0, 0, 0, 0)
        
    def set_widget(self, widget: QWidget, *args):
        self.native.layout().addWidget(widget, *args)
        widget.setParent(self.native)
        self.central_widget = widget

def magicwidget(qcls: type[QWidget]):
    @wraps(qcls.__init__)
    def __init__(self: FreeWidget, *args, **kwargs):
        FreeWidget.__init__(self)
        self._qwidget = qcls.__init__(*args, **kwargs)
        self.set_widget(self._qwidget)
    
    new_class = type(qcls.__name__, (FreeWidget,), {})
    new_class.__init__ = __init__
    return new_class


class NotInstalled:
    def __init__(self, msg):
        self.msg = msg
    
    def __getattr__(self, key: str):
        raise ModuleNotFoundError(self.msg)
    
    def __call__(self, *args, **kwargs):
        raise ModuleNotFoundError(self.msg)

==> magic-class-0.5.19.tar.gz: magic-class-0.5.19/magicclass/widgets/utils.py <==

from __future__ import annotations
from qtpy.QtWidgets import QWidget, QVBoxLayout, QGridLayout, QHBoxLayout
from magicgui.widgets import Widget
from magicgui.backends._qtpy.widgets import QBaseWidget


class _NotInitialized:
    def __init__(self, msg: str):
        self.msg = msg

    def __getattr__(self, key: str):
        raise RuntimeError(self.msg)


class FreeWidget(Widget):
    """A Widget class with any QWidget as a child."""
    
    _widget = _NotInitialized(
        "Widget is not correctly initialized. Must call `super().__init__` before using "
        "the widget."
        )
    
    def __init__(self, layout="vertical", **kwargs):
        super().__init__(widget_type=QBaseWidget, backend_kwargs={"qwidg": QWidget}, **kwargs)
        self.native: QWidget
        self.central_widget: QWidget
        if layout == "vertical":
            self.native.setLayout(QVBoxLayout())
        elif layout == "horizontal":
            self.native.setLayout(QHBoxLayout())
        elif layout == "grid":
            self.native.setLayout(QGridLayout())
        else:
            raise ValueError(layout)
        self.native.setContentsMargins(0, 0, 0, 0)


    def set_widget(self, widget: QWidget, *args):
        """Set the central widget to the widget."""
        self.native.layout().addWidget(widget, *args)
        widget.setParent(self.native)
        self.central_widget = widget

def magicwidget(qcls: type[QWidget]):
    from ..utils import iter_members
    for name, attr in iter_members(qcls):
        def _(self: FreeWidget, *args, **kwargs):
            return attr(self.central_widget, *args, **kwargs)
    cls = type(qcls.__name__, (FreeWidget, ), {})
